#region 1. Using Statemetns
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XnaAssignments;
#endregion

namespace MyGame_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XnaAssignmentBase
    /// </summary>
    public class MyGame : XnaAssignmentBase
    {
        #region Dimension of all objects
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = 0.0f;        // lower left corner is (0, 0)
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = 320f;        // width of the world 
        
        // since aspect ratio is 16:9, width of 320 means height will be 180
        // 
        // Kid size is:
        private const float KID_WIDTH = 20.0f;
        private const float KID_HEIGHT = 30.0f;

        // Bird size
        private const float BIRD_WIDTH = 30.0f;
        private const float BIRD_HEIGHT = 20.0f;

        // Egg size
        private const float EGG_WIDTH = 6.0f;
        private const float EGG_HEIGHT = 9.0f;

        // Umbrella size
        private const float UMB_WIDTH = 18.0f;
        private const float UMB_HEIGHT = 21.0f;

        private const float TOLERANCE = 10.0f;

        private const float UMB_SPEED = 5.0f;
        private const float EGG_SPEED = -2.0f;
        #endregion

        #region Constructor
        /// <summary>
        /// This is the constructor. Notice we must call the base class's constructor
        /// by passing in the coordinate of the lower-left corner, and the width of the world.
        /// </summary>
        public MyGame()
            : base(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH)
        {
            //  Notice:  memory allocation and initialization should be performed
            //           in InitializedWorld(). The constructor should be empty.
        }
        #endregion

        int m_BirdTouched;      // number of times bird touched by umbrella
        int m_KidTouched;       // number of times kid touched by egg

        Random m_Rand;

        Vector2 m_KidPos;        // Position of the kid
        
        Vector2 m_BirdPos;       // Position of the bird
        Vector2 m_BirdVelocity;  // Velocity of the bird

        Vector2 m_EggPos;        // Position of the Egg
        Vector2 m_EggVelocity;   // Egg's velocity
        bool m_EggDropped;       // Egg has been dropped

        Vector2 m_UmbrellaPos;   // Position of the umbrella
        Vector2 m_UmbVelocity;   // Velocity of the umbrella
        bool m_UmbShot;          // Umberella has been shot
        
        #region Initialization of the world
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            m_Rand = new Random();

            // DON'T FORGET TO INITIALIZE EVERYTHING HERE!!!

            XnaAssignmentBase.BackgroundColor = new Color(140, 140, 140);  // Gray-ish background
        }
        #endregion

        #region Update the game
        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            // Quits with the back button
            if (XnaAssignmentBase.GamePad.ButtonBackClicked())
                this.Exit();


            #region update the bird

            #endregion for updating the bird

            #region process the egg: random dropping
                if (m_Rand.NextDouble() > 0.1)
                {
                    // 50% of chance for dropping egg
                    // PlayACue(0.3f, "DroppedEgg");
                }
            #endregion

            #region Udpate the kid by the left-thumb stick (left/right arrow)
            // XnaAssignmentBase.GamePad.ThumbSticks.Left.X;

            #endregion 

            #region Shoots an umbrella with A-button (if one is not already shot)
            if (XnaAssignmentBase.GamePad.Buttons.A == ButtonState.Pressed)
            {
                PlayACue(0.4f, "ShootUmb");
            }
            #endregion 

            #region interact the umberall with the egg if both exists
            if (m_UmbShot && m_EggDropped)
            {
                // PlayACue(0.4f, "EggZapped");
            }
            #endregion 

            #region Now, umbrella chase the bird away 
            if (m_UmbShot)
            {
                // PlayACue(0.34f,"BirdChased");
            }
            #endregion

            #region Now, egg on the kid
            if (m_EggDropped)
            {
                // PlayACue(0.3f, "KidEgged");
            }
            #endregion
            // Notice one should never draw anything in the UpdateWorld() function!
        }
        #endregion

        #region Draw everything to the application window
        /// <summary>
        /// Draw the application state
        /// </summary>
        protected override void DrawWorld()
        {
            // Draw the Kid
            DrawRectangle(m_KidPos, KID_WIDTH, KID_HEIGHT, 0.0f, Color.White, Color.White, "L-Kid");
            DrawRectangle(m_BirdPos, BIRD_WIDTH, BIRD_HEIGHT, 0.0f, Color.White, Color.White, "L-Bird");
            
            if (m_EggDropped)
                DrawRectangle(m_EggPos, EGG_WIDTH, EGG_HEIGHT, 0.0f, Color.White, Color.White, "Egg");

            if (m_UmbShot)
                DrawRectangle(m_UmbrellaPos, UMB_WIDTH, UMB_HEIGHT, 0.0f, Color.White, Color.White, "Umbrella");

            EchoToTopStatus("Scores: Kid touched=" + m_KidTouched + "    Bird touched=" + m_BirdTouched);
            EchoToBottomStatus("CSS 290: Homework Assignment 4");
        }
        #endregion
    }
}
